"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
var binding = require('../binding');
/// <summary> Create a store with an id. </summary>
/// <param name="id"> String identifier which can be used to get the store from all isolates. </summary>
/// <returns> A store object or throws Error if store with this id already exists. </returns>
/// <remarks> Store object will be destroyed when reference from all isolates are unreferenced. 
/// It's usually a best practice to keep a long-living reference in user modules or global scope. </remarks>
function create(id) {
    return binding.createStore(id);
}
exports.create = create;
/// <summary> Get a store with an id. </summary>
/// <param name="id"> String identifier which is passed to create/getOrCreate earlier. </summary>
/// <returns> A store object if exists, otherwise undefined. </returns>
function get(id) {
    return binding.getStore(id);
}
exports.get = get;
/// <summary> Get a store with an id, or create it if not exist. </summary>
/// <param name="id"> String identifier which can be used to get the store from all isolates. </summary>
/// <returns> A store object associated with the id. </returns>
/// <remarks> Store object will be destroyed when reference from all isolates are unreferenced. 
/// It's usually a best practice to keep a long-living reference in user modules or global scope. </remarks>
function getOrCreate(id) {
    return binding.getOrCreateStore(id);
}
exports.getOrCreate = getOrCreate;
/// <summary> Returns number of stores that is alive. </summary>
function count() {
    return binding.getStoreCount();
}
exports.count = count;
