"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
/// <summary> Per-isolate cid => constructor registry. </summary>
var _registry = new Map();
/// <summary> Register a TransportableObject sub-class with a Constructor ID (cid). </summary>
function registerConstructor(cid, objectClass) {
    if (_registry.has(cid)) {
        throw new Error("Constructor ID (cid) \"" + cid + "\" is already registered.");
    }
    _registry.set(cid, objectClass);
}
exports.registerConstructor = registerConstructor;
/// <summary> Register a TransportableObject sub-class with a Constructor ID (cid). </summary>
function getConstructor(cid) {
    var objectClass = _registry.get(cid);
    if (objectClass == null) {
        throw new Error("Unrecognized Constructor ID (cid) \"" + cid + "\". Please ensure @cid is applied on the class or transport.register is called on the class.");
    }
    return objectClass;
}
exports.getConstructor = getConstructor;
/// <summary> Register a TransportableObject sub-class with a Constructor ID (cid). </summary>
function newInstance(cid) {
    var objectClass = _registry.get(cid);
    if (objectClass == null) {
        throw new Error("Unrecognized Constructor ID (cid) \"" + cid + "\". Please ensure @cid is applied on the class or transport.register is called on the class.");
    }
    var object = new objectClass();
    return object;
}
exports.newInstance = newInstance;
