"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
/// <summary> Represent option when the zone is recycled by GC. </summary>
var ZoneRecycleMode;
(function (ZoneRecycleMode) {
    /// <summary>
    /// When a Zone is no longer referenced and all tasks on its workers are completed,
    /// it will be recycled automatically.
    /// </summary>
    ZoneRecycleMode["AUTO"] = "auto";
    /// <summary>
    /// The zone will not exit automatically.
    /// User should call zone.recycle() manually.
    /// </summary>
    ZoneRecycleMode["MANUAL"] = "manual";
})(ZoneRecycleMode = exports.ZoneRecycleMode || (exports.ZoneRecycleMode = {}));
/// <summary> Default ZoneSettings </summary>
exports.DEFAULT_SETTINGS = {
    /// <summary> Set default workers to 2. </summary>
    workers: 2,
    /// <summary> Set default recycling option to AUTO. </summary>
    recycle: ZoneRecycleMode.AUTO
};
/// <summary> Represent option to transport arguments in zone.execute. </summary>
var TransportOption;
(function (TransportOption) {
    /// <summary> transport.marshall/unmarshall will be done by `napajs` automatically.
    /// This is the most common way, but may not be performance optimal with objects
    /// that will be shared in multiple zone.execute.
    /// </summary>
    TransportOption["AUTO"] = "auto";
    /// <summary> transport.marshall/unmarshall will be done by user manually. </summary>
    TransportOption["MANUAL"] = "manual";
})(TransportOption = exports.TransportOption || (exports.TransportOption = {}));
/// <summary> Default execution options. </summary>
exports.DEFAULT_CALL_OPTIONS = {
    /// <summary> No timeout. </summary>
    timeout: 0,
    /// <summary> Set argument transport option to automatic. </summary>
    transport: TransportOption.AUTO
};
